Package: network
Version: 1.13.0.1
Date: 2015-08-31
Title: Classes for Relational Data
Authors@R: c(
    person("Carter T.", "Butts", role=c("aut","cre"), email="buttsc@uci.edu"),
    person("David", "Hunter", role=c("ctb"), email="dhunter@stat.psu.edu"),
    person("Mark", "Handcock", role=c("ctb"), email="handcock@stat.ucla.edu"),
    person("Skye", "Bender-deMoll", role=c("ctb"), email="skyebend@uw.edu"),
    person("Jeffrey", "Horner", role=c("ctb"), email="jeffrey.horner@gmail.com"))
Depends: R (>= 2.10), utils
Suggests: sna, statnet.common, testthat
Description: Tools to create and modify network objects.  The network class can represent a range of relational data types, and supports arbitrary vertex/edge/graph attributes.
License: GPL (>= 2)
URL: http://statnet.org/
NeedsCompilation: yes
Packaged: 2018-04-02 12:10:58 UTC; hornik
Author: Carter T. Butts [aut, cre],
  David Hunter [ctb],
  Mark Handcock [ctb],
  Skye Bender-deMoll [ctb],
  Jeffrey Horner [ctb]
Maintainer: Carter T. Butts <buttsc@uci.edu>
Repository: CRAN
Date/Publication: 2018-04-02 20:05:25 UTC
Built: R 3.5.1; x86_64-w64-mingw32; 2018-10-19 03:10:39 UTC; windows
Archs: x64
