Package: network
Version: 1.17.2
Date: 2022-05-20
Title: Classes for Relational Data
Authors@R: c(
    person("Carter T.", "Butts", role=c("aut","cre"), email="buttsc@uci.edu"),
    person("David", "Hunter", role=c("ctb"), email="dhunter@stat.psu.edu"),
    person("Mark", "Handcock", role=c("ctb"), email="handcock@stat.ucla.edu"),
    person("Skye", "Bender-deMoll", role=c("ctb"), email="skyebend@uw.edu"),
    person("Jeffrey", "Horner", role=c("ctb"), email="jeffrey.horner@gmail.com"),
    person("Li", "Wang", role=c("ctb"), email="lxwang@uw.edu"),
    person("Pavel N.", "Krivitsky", role=c("ctb"), email="pavel@statnet.org", comment=c(ORCID="0000-0002-9101-3362")),
    person("Brendan", "Knapp", role=c("ctb"), email="brendan.g.knapp@gmail.com", comment=c(ORCID="0000-0003-3284-4972")),
    person("Michał", "Bojanowski", role=c("ctb"), email="mbojanowski@kozminski.edu.pl"),
    person("Chad", "Klumb", role=c("ctb"), email="cklumb@gmail.com"))
Author: Carter T. Butts [aut, cre],
  David Hunter [ctb],
  Mark Handcock [ctb],
  Skye Bender-deMoll [ctb],
  Jeffrey Horner [ctb],
  Li Wang [ctb],
  Pavel N. Krivitsky [ctb] (<https://orcid.org/0000-0002-9101-3362>),
  Brendan Knapp [ctb] (<https://orcid.org/0000-0003-3284-4972>),
  Michał Bojanowski [ctb],
  Chad Klumb [ctb]
Maintainer: Carter T. Butts <buttsc@uci.edu>
Depends: R (>= 2.10), utils
Imports: tibble, magrittr, statnet.common (>= 4.5), stats
Suggests: sna, testthat, covr
Description: Tools to create and modify network objects.  The network class can represent a range of relational data types, and supports arbitrary vertex/edge/graph attributes.
License: GPL (>= 2)
URL: http://statnet.org/
RoxygenNote: 7.1.1
Collate: 'access.R' 'as.edgelist.R' 'assignment.R' 'coercion.R'
        'constructors.R' 'dataframe.R' 'fileio.R' 'layout.R' 'misc.R'
        'network-package.R' 'operators.R' 'plot.R' 'printsum.R' 'zzz.R'
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2022-05-21 06:20:27 UTC; buttsc
Repository: CRAN
Date/Publication: 2022-05-21 07:40:02 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2022-05-21 12:17:17 UTC; windows
Archs: x64
